@@ -3,6 +3,11 @@
 Generate ROC, PR and Correlation curves.
 Format of input: Input format: CSV by --delimiter: len(rocord)==4. Ex: modelId, weight, score, label)
 """
+from __future__ import division
+from __future__ import print_function
+from builtins import str
+from builtins import zip
+from past.utils import old_div
 __author__ = 'zhdeng'
 
 import sys, os, time
@@ -100,13 +105,13 @@ def batchSort(input, output, key, bufferSize):
         os.remove(chunk.name)
       except Exception:
         pass
-  print "sorted file %s ready" % (output)
+  print("sorted file %s ready" % (output))
 
 def calculateAUC(rocPoints):
   AUC = 0.0
   lastPoint = (0, 0)
   for point in rocPoints:
-    AUC += (point[1] + lastPoint[1]) * (point[0] - lastPoint[0]) / 2
+    AUC += old_div((point[1] + lastPoint[1]) * (point[0] - lastPoint[0]), 2)
     lastPoint = point
   return AUC
 
@@ -155,17 +160,17 @@ def plotCurves(dataByModel):
   recallAx.set_title('Threshold score vs recall')
   recallAx.grid(True)
 
-  for (model, data) in dataByModel.items():
-    (recalls, precisions) = zip(*(data['PR']))
+  for (model, data) in list(dataByModel.items()):
+    (recalls, precisions) = list(zip(*(data['PR'])))
     prAx.plot(recalls, precisions, marker='o', linestyle='--', label=model)
 
-    (fallouts, recalls) = zip(*(data['ROC']))
+    (fallouts, recalls) = list(zip(*(data['ROC'])))
     rocAx.plot(fallouts, recalls, marker='o', linestyle='--', label=model)
 
-    (pCtrs, eCtrs) = zip(*(data['CORR']))
+    (pCtrs, eCtrs) = list(zip(*(data['CORR'])))
     corrAx.plot(pCtrs, eCtrs, label=model)
 
-    (score, recall, precision, fallout) = zip(*(data['cutoff']))
+    (score, recall, precision, fallout) = list(zip(*(data['cutoff'])))
 
     recallAx.plot(score, recall, label=model+'_recall')
     precisionAx.plot(score, precision, label=model+'_precision')
@@ -189,14 +194,14 @@ def plotCurves(dataByModel):
 
   pyplot.close()
   pngs = '{result}/pr_curve.png {result}/roc_curve.png {result}/corr_curve.png {result}/precision.png {result}/recall.png'.format(result=RESULT_DIR)
-  print 'png: ', pngs
+  print('png: ', pngs)
 
 def groupDataByModel(inputDir):
   """
   Group data to separated file by model.
   """
   t1 = time.time()
-  print "merging files by model to %s" % MERGE_DIR
+  print("merging files by model to %s" % MERGE_DIR)
   ensure_dir(MERGE_DIR)
   fileByModel = dict()
   for file in walktree(inputDir):
@@ -204,16 +209,16 @@ def groupDataByModel(inputDir):
       fields = line.split(options.delimiter)
       if options.ignoreInvalid:
         if len(fields) != 4 or fields[0] == '' or fields[1] == '' or fields[2] == '' or fields[3] == '':
-          print 'Ingonre Invalid line', fields
+          print('Ingonre Invalid line', fields)
           continue
       model = fields[0]
       if model not in fileByModel:
         fileByModel[model] = open('%s/%s.txt' % (MERGE_DIR, model), 'w')
       fileByModel[model].write(line)
-  for file in fileByModel.values():
+  for file in list(fileByModel.values()):
     file.close()
   t2 = time.time()
-  print "merging files take %ss" % (t2 - t1)
+  print("merging files take %ss" % (t2 - t1))
 
   if options.useMask:
     fileByModel = removeMaskData(fileByModel)
@@ -223,14 +228,14 @@ def removeMaskData(dataByName):
   toRemoveList = []
   masks = options.useMask.split(',')
   for mask in masks:
-    for name in dataByName.iterkeys():
+    for name in dataByName.keys():
       if mask.endswith('*'):
         if name.startswith(mask[:-1]):
-          print 'remove %s because of filter:%s' % (name, mask)
+          print('remove %s because of filter:%s' % (name, mask))
           toRemoveList.append(name)
       else:
         if name == mask:
-          print 'remove %s because of filter:%s' % (name, mask)
+          print('remove %s because of filter:%s' % (name, mask))
           toRemoveList.append(name)
 
   for removeName in toRemoveList:
@@ -254,10 +259,10 @@ def sortDataFileByModel(fileByModel):
   Use external sort to sort data file by score column.
   """
   t1 = time.time()
-  print "sorting files...."
+  print("sorting files....")
   ensure_dir(SORT_DIR)
   processPool = []
-  for model in fileByModel.keys():
+  for model in list(fileByModel.keys()):
     mergedFile = '%s/%s.txt' % (MERGE_DIR, model)
     sortedFile = '%s/%s.txt' % (SORT_DIR, model)
     if options.ignoreInvalid:
@@ -272,18 +277,18 @@ def sortDataFileByModel(fileByModel):
   for process in processPool:
     process.join()
   t2 = time.time()
-  print "sorting files take %ss" % (t2 - t1)
+  print("sorting files take %ss" % (t2 - t1))
 
 def processDataByModel(fileByModel):
   """
   Process data by model. Wait all the subprocess finish then plot curves together.
   """
   t1 = time.time()
-  print "processing data...."
+  print("processing data....")
   pool = Pool(len(fileByModel))
   dataByModel = dict()
   resultList = []
-  for model in fileByModel.keys():
+  for model in list(fileByModel.keys()):
     sortedFile = '%s/%s.txt' % (SORT_DIR, model)
     result = pool.apply_async(processData, args=(model, sortedFile))
     resultList.append(result)
@@ -291,32 +296,32 @@ def processDataByModel(fileByModel):
     try:
       (model, data) = result.get()
       dataByModel[model] = data
-    except Exception, e:
+    except Exception as e:
       if not options.ignoreInvalid:
         raise e
   t2 = time.time()
 
   if options.aucSelect:
     selectLimit = options.selectLimit
-    print 'Sort model by AUC and select top', selectLimit
-    sortedModelTuple = sorted(dataByModel.items(), key=lambda item: item[1]['AUC'], reverse=True)
+    print('Sort model by AUC and select top', selectLimit)
+    sortedModelTuple = sorted(list(dataByModel.items()), key=lambda item: item[1]['AUC'], reverse=True)
     dataByModel = dict(sortedModelTuple[:selectLimit])
 
   if options.verbose:
-    print dataByModel
+    print(dataByModel)
   plotCurves(dataByModel)
-  print "processing data take %ss" % (t2 - t1)
+  print("processing data take %ss" % (t2 - t1))
   
 def processData(model, input):
   """
   Process data. Bin data into options.shardCount bins. Accumulate data for each bin and populate necessary metrics. 
   """
-  print 'processData data for %s' % model
+  print('processData data for %s' % model)
   data = np.loadtxt(input, delimiter = options.delimiter,
     dtype = {'names' : ('model', 'weight', 'score', 'label'),
              'formats' : ('S16', 'f4', 'f4', 'i1')}) 
   dataSize = len(data)
-  shardSize = dataSize / options.shardCount
+  shardSize = old_div(dataSize, options.shardCount)
 
   rocPoints = [(0, 0)]
   prPoints = []
@@ -365,12 +370,12 @@ def processData(model, input):
       falsePositive += weight
 
     if partitionSize % shardSize == 0 or partitionSize == dataSize:
-      recall = truePositive / totalConditionPositive
-      fallout = falsePositive / totalConditionNegative;
-      precision = truePositive / (truePositive + falsePositive)
+      recall = old_div(truePositive, totalConditionPositive)
+      fallout = old_div(falsePositive, totalConditionNegative);
+      precision = old_div(truePositive, (truePositive + falsePositive))
 
-      meanPctr = binTotalScore / binWeight
-      eCtr = binPositive / binWeight
+      meanPctr = old_div(binTotalScore, binWeight)
+      eCtr = old_div(binPositive, binWeight)
 
       rocPoints += [(fallout, recall)]
       prPoints += [(recall, precision)]
@@ -387,14 +392,14 @@ def processData(model, input):
   cutoff = sorted(cutoff, key=lambda x:x[0])
 
   AUC = calculateAUC(rocPoints)
-  OER = truePositive / overallTatalScore  #Observed Expected Ratio
-  F1 = 2 * truePositive / (truePositive + falsePositive + totalConditionPositive)
+  OER = old_div(truePositive, overallTatalScore)  #Observed Expected Ratio
+  F1 = old_div(2 * truePositive, (truePositive + falsePositive + totalConditionPositive))
 
-  print '%s AUC: %f' % (model, AUC)
-  print '%s F1: %f' % (model, F1)
-  print '%s Observed/Expected Ratio: %f' % (model, OER)
+  print('%s AUC: %f' % (model, AUC))
+  print('%s F1: %f' % (model, F1))
+  print('%s Observed/Expected Ratio: %f' % (model, OER))
   if options.cutoff:
-    print '%s cutoff:' % model, cutoff
+    print('%s cutoff:' % model, cutoff)
 
   return model, { 'ROC': rocPoints, 'PR': prPoints, 'CORR': corrPoints, 'AUC': AUC, 'OER': OER, 'F1': F1, 'cutoff': cutoff}
 
@@ -410,7 +415,7 @@ def roc(inputDir, phrase):
   if phrase == PHRASE_GROUP:
     fileByModel = groupDataByModel(inputDir)
     if options.verbose:
-      print fileByModel
+      print(fileByModel)
     phrase = PHRASE_GROUP + 1
   else:
     fileByModel = loadFileNameByModel(MERGE_DIR)
@@ -452,14 +457,13 @@ def main():
   parser.add_option('--selectLimit', dest='selectLimit', default=0, type='int', help='Select top n model')
 
   (options,args) = parser.parse_args()
-  print options
-  print args
+  print(options)
+  print(args)
 
   if len(args) < 1:
-    print 'roc.py inputDir'
-    print 'Input format: CSV by --delimiter: len(rocord)==4. Ex: modelId, weight, score, label'
+    print('Input format: CSV by --delimiter: len(rocord)==4. Ex: modelId, weight, score, label')
     parser.print_help()
-    exit()
+    exit(1)
 
   roc(args[0], options.phrase)
 
